include "Port.inc"
include "ZeroPage.inc"
include "Vram.inc"
include "VVram.inc"
include "Chars.inc"

ext BackgroundPattern, SpritePattern
ext VVram_
ext HScroll_

PatternSize equ VramStep*CharHeight
WindowTop equ Vram+VramRowSize*2

dseg
Backup: public Backup
    defs WindowWidth*VVramHeight
SpriteLayer: public SpriteLayer
    defs VVramWidth*VVramHeight
MixedPattern: public MixedPattern
    defs PatternSize*4*SpriteCount

zseg
SpriteChar:
    defb 0
pMixedPattern:
    defw 0

pVVram equ ZW0
pVram equ ZW0
pSpriteLayer equ ZW1
pBackup equ ZW2
pSpritePattern equ ZW2
pPattern equ ZW3

yCount equ ZB0
cOffset equ ZB1
cyCount equ ZB2
cxCount equ ZB3
cpOffset equ ZB4
spOffset equ ZB5


cseg
InitVram: public InitVram
    lda #160 | sta LCD_X_Size
    lda #160 | sta LCD_Y_Size
    stz X_Scroll
    stz Y_Scroll

rts


; void ClearScreen();
cseg
ClearScreen_: public ClearScreen_
    pha
        lda #low VramOrg | sta pVram
        lda #high VramOrg | sta pVram+1
        ldx #$1e
        lda #0
        do
            ldy #0
            do
                sta (<pVram),y
                iny
            while ne | wend
            inc <pVram+1
            dex
        while ne | wend

        lda #low Backup | sta <pBackup
        lda #high Backup | sta <pBackup+1
        ldx #VVramHeight
        do
            ldy #0 | tya
            do
                sta (<pBackup),y
                iny
                cpy #WindowWidth
            while ne | wend
            lda <pBackup | clc|adc #WindowWidth | sta <pBackup
            lda <pBackup+1 | adc #0 | sta <pBackup+1
            dex
        while ne | wend
    pla
rts


cseg
Put: ;(,a)
    tax
        stz <pPattern+1
        asl a | rol <pPattern+1 ;*2
        asl a | rol <pPattern+1 ;*4
        asl a | rol <pPattern+1 ;*8
        asl a | rol <pPattern+1 ;*16
        sta <pPattern
    txa
    cmp #Char_End
    if cc
        lda <pPattern | clc|adc #low BackgroundPattern | sta <pPattern
        lda <pPattern+1 | adc #high BackgroundPattern | sta <pPattern+1
    else
        lda <pPattern | clc|adc #low(MixedPattern-PatternSize*Char_End) | sta <pPattern
        lda <pPattern+1 | adc #high(MixedPattern-PatternSize*Char_End) | sta <pPattern+1
    endif
    
    ldx #CharHeight
    do
        ldy #0
        do
            lda (<pPattern),y
            sta (<pVram),y
            iny
            cpy #VramStep
        while ne | wend
        lda <pPattern | clc|adc #VramStep | sta <pPattern
        lda <pPattern+1 | adc #0 | sta <pPattern+1

        lda <pVram | clc|adc #VramWidth | sta <pVram
        lda <pVram+1 | adc #0 | sta <pVram+1

        dex
    while ne | wend
    lda <pVram | clc|adc #low(VramStep-VramWidth*CharHeight) | sta <pVram
    lda <pVram+1 | adc #high(VramStep-VramWidth*CharHeight) | sta <pVram+1
rts


; ptr<byte> Put(ptr<byte> pVram, byte c);
dseg
Put_@Param0: public Put_@Param0
Put_pVram:
PrintC_@Param0: public PrintC_@Param0
PrintC_pVram:
    defw 0
Put_@Param1: public Put_@Param1
Put_c:
PrintC_@Param1: public PrintC_@Param1
PrintC_c:
    defb 0
cseg
Put_: public Put_
    pha
        lda Put_pVram | sta <pVram
        lda Put_pVram+1 | sta <pVram+1
        lda Put_c
        jsr Put

        lda Put_pVram | clc|adc #VramStep | tay
        lda Put_pVram+1 | adc #0 | tax
    pla
rts


; ptr<byte> PrintC(ptr<byte> pVram, byte c);
cseg
CharTable:
    defb " -0123456789ABCEFGHIMNOPRSTUV"
PrintC_: public PrintC_
    pha
        lda PrintC_pVram | sta <pVram
        lda PrintC_pVram+1 | sta <pVram+1
        
        lda PrintC_c
        ldx #0
        do
            cmp CharTable,x
            beq PrintC_break
            inx
            cpx #29
        while ne | wend
        PrintC_break:
        txa
        jsr Put

        lda PrintC_pVram | clc|adc #VramStep | tay
        lda PrintC_pVram+1 | adc #0 | tax
    pla
rts


; void BeginDraw();
cseg
BeginDraw_: public BeginDraw_
    pha
        lda #Char_End | sta <SpriteChar
        lda #low MixedPattern | sta <pMixedPattern
        lda #high MixedPattern | sta <pMixedPattern+1

        lda #low VVram_| sta <pVVram
        lda #high VVram_ | sta <pVVram+1

        lda #low SpriteLayer | sta <pSpriteLayer
        lda #high SpriteLayer | sta <pSpriteLayer+1
        
        ldx #VVramHeight
        do
            ldy #0
            do
                lda (<pVVram),y | sta (<pSpriteLayer),y
                iny
                cpy #VVramWidth
            while ne | wend
            lda <pVVram | clc|adc #low VVramWidth | sta <pVVram
            lda <pVVram+1 | adc #0 | sta <pVVram+1

            lda <pSpriteLayer | clc|adc #VVramWidth | sta <pSpriteLayer
            lda <pSpriteLayer+1 | adc #0 | sta <pSpriteLayer+1

            dex
        while ne | wend
   pla
rts


; void EndDraw();
cseg
EndDraw_: public EndDraw_
    pha
        lda #low WindowTop | sta <pVram
        lda #high WindowTop | sta <pVram+1

        lda HScroll_ | clc|adc #low SpriteLayer  | sta <pSpriteLayer
        lda #high SpriteLayer | adc #0 | sta <pSpriteLayer+1
        
        lda #low Backup | sta <pBackup
        lda #high Backup | sta <pBackup+1

        lda #VVramHeight | sta <yCount
        do
            ldy #0
            do
                lda (<pSpriteLayer),y
                cmp #Char_End
                bcs EndDraw_write
                cmp (<pBackup),y
                if ne
                    EndDraw_write:
                    sta (<pBackup),y
                    phy
                        jsr Put
                    ply
                else
                    lda <pVram | clc|adc #VramStep | sta <pVram
                    lda <pVram+1 | adc #0 | sta <pVram+1
                endif
                iny
                cpy #WindowWidth
            while ne | wend
            lda <pVram | clc|adc #low(VramRowSize-VramStep*WindowWidth) | sta <pVram
            lda <pVram+1 | adc #high(VramRowSize-VramStep*WindowWidth) | sta <pVram+1

            lda <pSpriteLayer | clc|adc #VVramWidth | sta <pSpriteLayer
            lda <pSpriteLayer+1 | adc #0 | sta <pSpriteLayer+1

            lda <pBackup | clc|adc #WindowWidth | sta <pBackup
            lda <pBackup+1 | adc #0 | sta <pBackup+1

            dec <yCount
        while ne | wend
    pla
rts


; void DrawSprite(byte x, byte y, byte pattern);
dseg
DrawSprite_@Param0: public DrawSprite_@Param0
DrawSprite_x:
    defb 0
DrawSprite_@Param1: public DrawSprite_@Param1
DrawSprite_y:
    defb 0
DrawSprite_@Param2: public DrawSprite_@Param2
DrawSprite_pattern:
    defb 0
cseg
SYTable:
defw SpriteLayer+VVramWidth*0,SpriteLayer+VVramWidth*1,SpriteLayer+VVramWidth*2,SpriteLayer+VVramWidth*3,SpriteLayer+VVramWidth*4,SpriteLayer+VVramWidth*5,SpriteLayer+VVramWidth*6,SpriteLayer+VVramWidth*7,SpriteLayer+VVramWidth*8,SpriteLayer+VVramWidth*9,SpriteLayer+VVramWidth*10,SpriteLayer+VVramWidth*11,SpriteLayer+VVramWidth*12,SpriteLayer+VVramWidth*13,SpriteLayer+VVramWidth*14,SpriteLayer+VVramWidth*15
DrawSprite_: public DrawSprite_
    pha
        lda DrawSprite_y | asl a | tax

        lda SYTable,x | sta <pSpriteLayer
        lda SYTable+1,x | sta <pSpriteLayer+1
        lda DrawSprite_x
        clc|adc <pSpriteLayer | sta <pSpriteLayer
        lda <pSpriteLayer+1 | adc #0 | sta <pSpriteLayer+1

        lda DrawSprite_pattern | stz <pSpritePattern
        lsr a | ror <pSpritePattern ;/2
        sta <pSpritePattern+1
        lda <pSpritePattern | clc|adc #low SpritePattern | sta <pSpritePattern
        lda <pSpritePattern+1 | adc #high SpritePattern | sta <pSpritePattern+1

        ldy #0 | sty <spOffset
        lda #2 | sta <yCount
        do
            ldy #0 | sty <cOffset
            do
                ldy <cOffset
                lda (<pSpriteLayer),y
                stz <pPattern+1
                asl a | rol <pPattern+1 ;*2
                asl a | rol <pPattern+1 ;*4
                asl a | rol <pPattern+1 ;*8
                asl a | rol <pPattern+1 ;*16
                clc|adc #low BackgroundPattern | sta <pPattern
                lda <pPattern+1 | adc #high BackgroundPattern | | sta <pPattern+1

                lda <SpriteChar
                sta (<pSpriteLayer),y
                inc a
                sta <SpriteChar

                ldy #0 | sty <cpOffset
                lda #CharHeight | sta <cyCount
                do
                    lda #VramStep | sta <cxCount
                    do
                        ldy <spOffset
                        lda (<pSpritePattern),y | iny
                        sty <spOffset
                        eor #$ff
                        ldy <cpOffset | and (<pPattern),y
                        ldy <spOffset
                        ora (<pSpritePattern),y | iny
                        sty <spOffset
                        ldy <cpOffset
                        sta (<pMixedPattern),y | iny
                        sty <cpOffset

                        dec <cxCount
                    while ne | wend
                    lda <spOffset | clc|adc #4 | sta <spOffset

                    dec <cyCount
                while ne | wend
                lda <pMixedPattern | clc|adc #PatternSize | sta <pMixedPattern
                lda <pMixedPattern+1 | adc #0 | sta <pMixedPattern+1

                lda <spOffset | clc|adc #4-64 | sta <spOffset

                ldy <cOffset | iny | sty <cOffset
                cpy #2
            while ne | wend
            lda <pSpriteLayer | clc|adc #VVramWidth | sta <pSpriteLayer
            lda <pSpriteLayer+1 | adc #0 | sta <pSpriteLayer+1

            lda <cOffset | clc|adc #VVramWidth-2 | sta <cOffset
            lda <spOffset | clc|adc #64-8 | sta <spOffset

            dec <yCount
        while ne | wend
    pla
rts
